@@ -1,5 +1,17 @@
 import contextlib
 import sys
+import os
+import json
+import logging
+import pandas as pd
+
+from typing import Union
+from os import path
+
+from rdkit import RDLogger
+
+RDLogger.DisableLog("rdApp.*")
+logging.getLogger().setLevel(logging.ERROR)
 
 if sys.version_info[:2] >= (3, 8):
     # TODO: Import directly (no need for conditional) when `python_requires = >= 3.8`
@@ -75,21 +87,6 @@ for imp in _OPTIONAL_IMPORTS_FOR_OCE_ONLINE:
         sys.modules[imp] = MagicMock()
         _mocked_imports.append(imp)
 
-from rdkit import RDLogger
-
-RDLogger.DisableLog("rdApp.*")
-
-import logging
-
-logging.getLogger().setLevel(logging.ERROR)
-
-import os
-from os import path
-
-import json
-
-import pandas as pd
-
 if not path.exists(path.join(path.expanduser("~"), f".oce/")):
     os.mkdir(path.join(path.expanduser("~"), f".oce/"))
     
@@ -110,12 +107,15 @@ defaults = {
     "CACHE": True
 }
 
+CONFIG_modified = False
 for k, v in defaults.items():
     if k not in CONFIG_:
         CONFIG_[k] = v
-        
-with open(CONFIG_PATH, "w+") as f:
-    json.dump(CONFIG_, f)
+        CONFIG_modified = True
+
+if CONFIG_modified:
+    with open(CONFIG_PATH, "w+") as f:
+        json.dump(CONFIG_, f)
 
 CONFIG = CONFIG_.copy()
 
@@ -140,11 +140,7 @@ def update_config():
         CONFIG["DEVICE"] = torch.device(CONFIG["MAP_LOCATION"])
         CONFIG["USE_CUDA"] = "cuda" in CONFIG["MAP_LOCATION"]
 
-    with open(CONFIG_PATH, "w+") as f:
-        json.dump(CONFIG_, f)
-
-
-def set_config_param(param, value):
+def set_config_param(param: str, value: Union[str, int, float, bool]):
     """Set a configuration parameter.
 
     Parameters:
@@ -153,9 +149,12 @@ def set_config_param(param, value):
     """
     CONFIG_[param] = value
     update_config()
+    
+    with open(CONFIG_PATH, "w+") as f:
+        json.dump(CONFIG_, f)
 
 
-def remove_config_param(param):
+def remove_config_param(param: str):
     """Remove a configuration parameter.
 
     Parameters:
@@ -163,7 +162,25 @@ def remove_config_param(param):
     """
     CONFIG_.pop(param)
     update_config()
+    
+    with open(CONFIG_PATH, "w+") as f:
+        json.dump(CONFIG_, f)
 
+def create_config_default_param(param: str, value: Union[str, int, float, bool]):
+    """Create a default configuration parameter.
+
+    Parameters:
+        param: the parameter to create.
+        value: the value to set the parameter to.
+    """
+    
+    if param not in CONFIG_:
+        CONFIG_[param] = value
+        
+    update_config()
+    
+    with open(CONFIG_PATH, "w+") as f:
+        json.dump(CONFIG_, f)
 
 update_config()
 
@@ -173,11 +190,9 @@ def ExampleDataFrame():
         "https://storage.googleapis.com/oloren-public-data/sample-csvs/sample_data3.csv"
     )
 
-
 from .base_class import *
 from .basics import *
 from .beta import *
-from .benchmarks import *
 from .dataset import *
 from .ensemble import *
 from .external import *
@@ -252,31 +267,19 @@ def MISSING_DEPENDENCIES():
 
 if len(_mocked_imports) > 0:
     import textwrap
-    message = f"Some missing imports detected: {(','.join(_mocked_imports)[:30])}..." + \
-    f"Automatically triggering online mode, where all OCE code is compiled locally and sent to AWS for execution." + \
-    f"If you would like to use OCE online mode with private data or hosted on your own infrastructure, email." + \
-    f" contact@oloren.ai for enterprise offerings." + \
-    f"To exit online mode, run oce.online_session.__exit__()"
-
-    message = f"""
-Some missing imports detected: {(','.join(_mocked_imports)[:30])}...
-Automatically triggering online mode, where all OCE code is compiled locally
-and sent to AWS for execution. If you would like to use OCE online mode with private
-data or hosted on your own infrastructure, email contact@oloren.ai for enterprise
-offerings. To exit online mode, run olorenchemengine.online_session.__exit__()
-""".replace("\n", " ")
 
     message = f"""
         To complete installation of Oloren ChemEngine, either:
-            (1) Run oce.online() to use the demonstration package, Oloren ChemEngine Online,
-            (2) Install the missing dependencies, instructions can be found oce.MISSING_DEPENDENCIES(), or
-            (3) Email contact@oloren.ai with subject "Oloren ChemEngine Enterprise", for a secure privately hosted
+            (1) Install the missing dependencies, these can be found by calling oce.MISSING_DEPENDENCIES(), or
+            (2) Email contact@oloren.ai with subject "Oloren ChemEngine Enterprise", for a secure privately hosted
                 Server version.
+            (3) Run oce.online() to use the demonstration package, Oloren ChemEngine Online,
+                Oloren ChemEngine online is the public, dependency-free version of OCE, which compiles OCE code locally for fast,
+                parallelized, remote execution on Oloren's cloud solution. Oloren ChemEngine online SHOULD NOT BE USED FOR
+                CONFIDENTAIL DATA, and is only intended for demonstration purposes. The securely privately hosted Server version is
+                called Oloren ChemEngine Enterprise.
 
-        Oloren ChemEngine online is the public, dependency-free version of OCE, which compiles OCE code locally for fast,
-        parallelized, remote execution on Oloren's cloud solution. Oloren ChemEngine online SHOULD NOT BE USED FOR
-        CONFIDENTAIL DATA, and is only intended for demonstration purposes. The securely privately hosted Server version is
-        called Oloren ChemEngine Enterprise.\n"""
+        \n"""
 
     # print(textwrap.fill(textwrap.dedent(message).replace("\n", " ").replace("    (", "\n     ("), 80))
     print(textwrap.dedent(message))
