@@ -12,6 +12,7 @@ from typing import Any, Callable, Tuple, Union
 
 import numpy as np
 import pandas as pd
+
 from sklearn.metrics import (
     average_precision_score,
     explained_variance_score,
@@ -24,6 +25,7 @@ from sklearn.metrics import (
 
 import olorenchemengine as oce
 from olorenchemengine.internal import *
+from olorenchemengine.dataset import *
 
 # List of metrics that can be used for evaluating classification models
 classification_metrics = {
@@ -60,271 +62,6 @@ metric_direction = {
     "Spearman": "higher",
 }
 
-
-class BaseObject(BaseClass):
-    """BaseObject is the parent class for all classes which directly wrap some object to be saved via joblib.
-
-    Attributes:
-        obj (object): the object which is wrapped by the BaseObject
-    """
-
-    @log_arguments
-    def __init__(self, obj=None):
-        self.obj = obj
-
-    def _save(self):
-        import joblib
-
-        b = io.BytesIO()
-        joblib.dump(self.obj, b)
-        return {"obj": b.getvalue()}
-
-    def _load(self, d):
-        import joblib
-
-        super()._load(d)
-        self.obj = joblib.load(io.BytesIO(d["obj"]))
-
-
-class BaseEstimator(BaseObject):
-
-    """Utility class used to wrap any object with a fit and predict method"""
-
-    def fit(self, X, y):
-        """Fit the estimator to the data
-
-        Parameters:
-            X (np.array): The data to fit the estimator to
-            y (np.array): The target data to fit the estimator to
-        Returns:
-            self (object): The estimator object fit to the data
-        """
-
-        return self.obj.fit(X, y)
-
-    def predict(self, X):
-        """Predict the output of the estimator
-
-        Parameters:
-            X (np.array): The data to predict the output of the estimator on
-        Returns:
-            y (np.array): The predicted output of the estimator
-        """
-        pred = self.obj.predict(X)
-        pred = np.array(pred)
-        if pred.ndim == 1:
-            pred = pred
-        else:
-            pred = np.array([pred])
-        return pred
-
-
-class LinearRegression(BaseEstimator):
-
-    """Wrapper for sklearn LinearRegression"""
-
-    @log_arguments
-    def __init__(self, *args, **kwargs):
-        from sklearn.linear_model import LinearRegression as LinearRegression_
-
-        self.obj = LinearRegression_(*args, **kwargs)
-
-
-class BasePreprocessor(BaseObject):
-    """BasePreprocessor is the parent class for all preprocessors which transform the features or properties of a dataset.
-
-    Methods:
-        fit: fit the preprocessor to the dataset
-        fit_transform: fit the preprocessor to the dataset return the transformed values
-        transform: return the transformed values
-        inverse_transform: return the original values from the transformed values
-    """
-
-    def fit(self, X):
-        """Fits the preprocessor to the dataset.
-
-        Parameters:
-            X (np.ndarray): the dataset
-
-        Returns:
-            The fit preprocessor instance
-        """
-        X = np.array(X)
-        if len(X.shape) == 1:
-            X = X.reshape(-1, 1)
-        return self.obj.fit(X)
-
-    def fit_transform(self, X):
-        """Fits the preprocessor to the dataset and returns the transformed values.
-
-        Parameters:
-            X (np.ndarray): the dataset
-
-        Returns:
-            The transformed values of the dataset as a numpy array
-        """
-        X = np.array(X)
-        if len(X.shape) == 1:
-            X = X.reshape(-1, 1)
-        X = self.obj.fit_transform(X)
-        return X
-
-    def transform(self, X):
-        """Returns the transformed values of the dataset as a numpy array.
-
-        Parameters:
-            X (np.ndarray): the dataset
-
-        Returns:
-            The transformed values of the dataset as a numpy array
-        """
-        X = np.array(X)
-        if len(X.shape) == 1:
-            X = X.reshape(-1, 1)
-        X = self.obj.transform(X)
-        return X
-
-    def inverse_transform(self, X):
-        """Returns the original values from the transformed values.
-
-        Parameters:
-            X (np.ndarray): the transformed values
-
-        Returns:
-            The original values from the transformed values
-        """
-        if len(X.shape) == 1:
-            X = X.reshape(-1, 1)
-        X = np.array(X)
-        return self.obj.inverse_transform(X)
-
-
-class QuantileTransformer(BasePreprocessor):
-    """QuantileTransformer is a BasePreprocessor which transforms a dataset by quantile transformation to specified distribution.
-
-    Attributes:
-        obj (sklearn.preprocessing.QuantileTransformer): the object which is wrapped by the BasePreprocessor
-    """
-
-    @log_arguments
-    def __init__(
-        self,
-        n_quantiles=1000,
-        output_distribution="normal",
-        subsample=1e5,
-        random_state=None,
-    ):
-
-        from sklearn.preprocessing import QuantileTransformer
-
-        self.obj = QuantileTransformer(
-            n_quantiles=n_quantiles,
-            output_distribution=output_distribution,
-            subsample=subsample,
-            random_state=random_state,
-        )
-
-
-class StandardScaler(BasePreprocessor):
-    """StandardScaler is a BasePreprocessor which standardizes the data by removing the mean and scaling to unit variance.
-
-    Attributes:
-        obj (sklearn.preprocessing.StandardScaler): the object which is wrapped by the BasePreprocessor
-    """
-
-    @log_arguments
-    def __init__(self, with_mean=True, with_std=True):
-
-        from sklearn.preprocessing import StandardScaler
-
-        self.obj = StandardScaler(with_mean=with_mean, with_std=with_std)
-
-
-class LogScaler(BasePreprocessor):
-    """LogScaler is a BasePreprocessor which standardizes the data by taking the log and then removing the mean and scaling to unit variance."""
-
-    @log_arguments
-    def __init__(self, with_mean=True, with_std=True):
-
-        from sklearn.preprocessing import StandardScaler
-
-        self.obj = StandardScaler(with_mean=with_mean, with_std=with_std)
-
-    def fit(self, X):
-        """Fits the preprocessor to the dataset.
-
-        Parameters:
-            X (np.ndarray): the dataset
-
-        Returns:
-            The fit preprocessor instance
-        """
-        X = np.array(X)
-        self.mean = np.mean(X)
-        X = np.log10(X + self.mean + 1e-3)
-        return self.obj.fit(X.reshape(-1, 1))
-
-    def fit_transform(self, X):
-        """Fits the preprocessor to the dataset and returns the transformed values.
-
-        Parameters:
-            X (np.ndarray): the dataset
-
-        Returns:
-            The transformed values of the dataset as a numpy array
-        """
-        X = np.array(X)
-        self.mean = np.mean(X)
-        X = np.log10(X + self.mean + 1e-3)
-        return self.obj.fit_transform(X.reshape(-1, 1)).reshape(-1)
-
-    def transform(self, X):
-        """Returns the transformed values of the dataset as a numpy array.
-
-        Parameters:
-            X (np.ndarray): the dataset
-
-        Returns:
-            The transformed values of the dataset as a numpy array
-        """
-        X = np.log10(np.array(X) + self.mean + 1e-3)
-        return self.obj.transform(X.reshape(-1, 1)).reshape(-1)
-
-    def inverse_transform(self, X):
-        """Returns the original values from the transformed values.
-
-        Parameters:
-            X (np.ndarray): the transformed values
-
-        Returns:
-            The original values from the transformed values
-        """
-        X = np.array(X)
-        return (
-            10 ** (self.obj.inverse_transform(X.reshape(-1, 1)).reshape(-1))
-            - self.mean
-            - 1e-3
-        )
-
-    def _save(self):
-        d = super()._save()
-        d["mean"] = self.mean
-        return d
-
-    def _load(self, d):
-        super()._load(d)
-        self.mean = d["mean"]
-        return self
-
-
-def detect_setting(data):
-    values, _ = np.unique(data, return_counts=True)
-    if len(values) <= 2:
-        return "classification"
-    else:
-        return "regression"
-
-
 class BaseModel(BaseClass):
     """
     BaseModel for training and evaluating different models
@@ -626,9 +363,8 @@ class BaseModel(BaseClass):
         self,
         X: Union[pd.DataFrame, np.ndarray],
         y: Union[pd.Series, list, np.ndarray],
-        n_splits: int = 5,
+        kf: BaseKFold = RandomKFold(n_splits = 5),
         error_model: BaseErrorModel = None,
-        ci: float = 0.8,
         scoring: str = None,
         **kwargs
     ):
@@ -654,17 +390,24 @@ class BaseModel(BaseClass):
         Returns:
             list: Cross validation metrics for each split
         """
-
+        
         self.fit(X, y, error_model=error_model)
 
+        X = np.array(X).flatten()
+        y = np.array(y).flatten()
+                
+        dataset = BaseDataset(data = pd.DataFrame({"X": SMILESRepresentation().convert(X),
+                                                   "y": y}).to_csv(),
+                              structure_col = "X",
+                              property_col = "y")
+        dataset = dataset + kf
+
         residuals = None
         scores = None
         pred = None
         true = None
 
         cross_val_metrics = []
-        X = np.array(X).flatten()
-        y = np.array(y).flatten()
 
         if scoring is None:
             if self.setting == "regression":
@@ -675,13 +418,9 @@ class BaseModel(BaseClass):
         from sklearn.calibration import calibration_curve
         from sklearn.model_selection import KFold
 
-        kf = KFold(n_splits=n_splits)
-
-        split = 1
-        for train_index, test_index in kf.split(X):
-            print('evaluating split {} of {}'.format(split, n_splits))
-            split += 1
-
+        for i in range(kf.get_n_splits()):
+            train_index = (dataset.data["cv"] == i+1)
+            test_index = (dataset.data["cv"] != i+1)
             X_train, X_test = X[train_index], X[test_index]
             y_train, y_test = y[train_index], y[test_index]
             model = self.copy()
@@ -723,20 +462,21 @@ class BaseModel(BaseClass):
                 else:
                     pred = np.concatenate((pred, prob_pred))
                     true = np.concatenate((true, prob_true))
-
             cross_val_metrics.append(metric_functions[scoring](y_test, y_pred_test))
 
         if self.setting == "regression":
-            self.calibrator = LinearRegression()
-            self.calibrator.fit(pred.reshape(-1, 1), true.reshape(-1, 1))
-            pred = self.calibrator.predict(pred.reshape(-1, 1)).reshape(-1)
-            true = true.reshape(-1)
-            residuals = np.abs(pred - true)
+            # self.calibrator = LinearRegression()
+            # self.calibrator.fit(pred.reshape(-1, 1), true.reshape(-1, 1))
+            # pred = self.calibrator.predict(pred.reshape(-1, 1)).reshape(-1)
+            # true = true.reshape(-1)
+            residuals = pred - true
             if hasattr(self, "error_model"):
+                self.error_model.build(self, X, y)
                 self.error_model._fit(residuals, scores, **kwargs)
         elif self.setting == "classification":
-            self.calibrator = LinearRegression()
-            self.calibrator.fit(pred.reshape(-1, 1), true.reshape(-1, 1))
+            # self.calibrator = LinearRegression()
+            # self.calibrator.fit(pred.reshape(-1, 1), true.reshape(-1, 1))
+            pass
 
         return cross_val_metrics
 
@@ -876,9 +616,7 @@ class BaseModel(BaseClass):
         if "error_model" in d.keys():
             error_model = loads(d["error_model"])
             error_model.model = self
-            print("Rebuilding error model...")
             error_model._build()
-            print("Refitting error model...")
             error_model._fit(error_model.residuals, error_model.scores)
             self.error_model = error_model
         if "normalization" in d.keys():
@@ -1167,6 +905,8 @@ class BaseErrorModel(BaseClass):
         self.bins = bins
         self.curvetype = curvetype
         
+        self._fit_regression_params = {}
+        
     def build(
         self,
         model: BaseModel,
@@ -1243,7 +983,7 @@ class BaseErrorModel(BaseClass):
             model.fit(X_train, y_train)
             
             y_pred_test = np.array(model.predict(X_test)).flatten()
-            pred_error = np.abs(y_test - y_pred_test)
+            pred_error = y_test - y_pred_test
             if residuals is None:
                 residuals = pred_error
             else:
@@ -1282,7 +1022,7 @@ class BaseErrorModel(BaseClass):
 
         self._upper_reg, X_upper, y_upper = self._fit_regression(0.5 + self.ci / 2)
         self._lower_reg, X_lower, y_lower = self._fit_regression(0.5 - self.ci / 2)
-        self.reg = lambda X: list(zip(self._lower_reg(X), self._upper_reg(X)))
+        self.reg = lambda X: [(min(l,0), max(u, 0)) for l, u in zip(self._lower_reg(X), self._upper_reg(X))]
         
         import plotly.express as px
         import plotly.graph_objects as go
@@ -1305,70 +1045,84 @@ class BaseErrorModel(BaseClass):
 
         return fig
 
-    def _fit_regression(self, quantile):
+    def _fit_regression(self, quantile, mode: str = None):
         """Fits a regression curve to a quantile of the fitting data.
 
         Args:
             quantile (float): fraction of data below the regression function
+            mode (str): the name of the fit either "upper" or "lower". This 
+                selection is used to determine if the regression fit has been
+                precomputed and if so, to return the precomputed fit.
 
         Returns:
             X and y coordinates for regression and regression function
         """
-        if self.method == "bin":
-            bins = min(len(np.unique(self.scores)), self.bins)
-            bin_labels = pd.cut(self.scores, bins, labels=False)
-            labels = np.unique(bin_labels)
-            X = [np.mean(self.scores[bin_labels == i]) for i in labels]
-            y = [np.quantile(self.residuals[bin_labels == i], quantile) for i in labels]
-        elif self.method == "qbin":
-            bins = min(len(np.unique(self.scores)), self.bins)
-            bin_labels = pd.qcut(self.scores, bins, labels=False, duplicates="drop")
-            labels = np.unique(bin_labels)
-            X = [np.mean(self.scores[bin_labels == i]) for i in labels]
-            y = [np.quantile(self.residuals[bin_labels == i], quantile) for i in labels]
-        elif self.method == "roll":
-            window = min(len(self.scores), self.window)
-            idxs = np.argsort(self.scores)
-            X = pd.Series(self.scores[idxs]).rolling(window).mean()[window-1:]
-            y = pd.Series(self.residuals[idxs]).rolling(window).quantile(quantile)[window-1:]
-        else:
-            raise NameError("Method {} is not recognized. Valid inputs are \"bin\", \"qbin\", and \"roll\".".format(self.method))
         
-        X = np.array(X)
-        y = np.array(y)
-
-        if len(X) == 1:
-            print("WARNING: Only one unique score computed. Please decrease bin/window size or use a larger dataset.")
-            return np.vectorize(lambda x: y[0] if (x == X[0]) else np.nan), X, y
-
-        from scipy.optimize import curve_fit
+        if not mode in self._fit_regression_params:
+            if self.method == "bin":
+                bins = min(len(np.unique(self.scores)), self.bins)
+                bin_labels = pd.cut(self.scores, bins, labels=False)
+                labels = np.unique(bin_labels)
+                X = [np.mean(self.scores[bin_labels == i]) for i in labels]
+                y = [np.quantile(self.residuals[bin_labels == i], quantile) for i in labels]
+            elif self.method == "qbin":
+                bins = min(len(np.unique(self.scores)), self.bins)
+                bin_labels = pd.qcut(self.scores, bins, labels=False, duplicates="drop")
+                labels = np.unique(bin_labels)
+                X = [np.mean(self.scores[bin_labels == i]) for i in labels]
+                y = [np.quantile(self.residuals[bin_labels == i], quantile) for i in labels]
+            elif self.method == "roll":
+                window = min(len(self.scores), self.window)
+                idxs = np.argsort(self.scores)
+                X = pd.Series(self.scores[idxs]).rolling(window).mean()[window-1:]
+                y = pd.Series(self.residuals[idxs]).rolling(window).quantile(quantile)[window-1:]
+            else:
+                raise NameError("Method {} is not recognized. Valid inputs are \"bin\", \"qbin\", and \"roll\".".format(self.method))
         
-        if self.curvetype == "auto":
-            min_mse = None
-            for reg in regression_functions:
-                func = regression_functions[reg]
-                init = regression_init[reg](X, y)
+            X = np.array(X)
+            y = np.array(y)
+
+            if len(X) == 1:
+                print("WARNING: Only one unique score computed. Please decrease bin/window size or use a larger dataset.")
+                return np.vectorize(lambda x: y[0] if (x == X[0]) else np.nan), X, y
+
+            from scipy.optimize import curve_fit
+        
+            if self.curvetype == "auto":
+                min_mse = None
+                for reg in regression_functions:
+                    func = regression_functions[reg]
+                    init = regression_init[reg](X, y)
+                    try:
+                        params = curve_fit(func, X, y, p0=init, maxfev = 500 * len(init))[0]
+                        y_pred = func(X, *params)
+                        mse = mean_squared_error(y, y_pred)
+                    except (RuntimeError, TypeError, ValueError):
+                        continue
+                    if min_mse is None or mse < min_mse:
+                        opt_func = func
+                        opt_params = params
+                        min_mse = mse
+                        used_curvetype = reg
+                assert min_mse is not None, "Curve fit failed to find a suitable regression model."
+            else:
+                opt_func = regression_functions[self.curvetype]
+                used_curvetype = self.curvetype
+                init = regression_init[self.curvetype](X, y)
                 try:
-                    params = curve_fit(func, X, y, p0=init, maxfev = 500 * len(init))[0]
-                    y_pred = func(X, *params)
-                    mse = mean_squared_error(y, y_pred)
+                    opt_params = curve_fit(opt_func, X, y, p0=init, maxfev = 500 * len(init))[0]
                 except (RuntimeError, TypeError, ValueError):
-                    continue
-                if min_mse is None or mse < min_mse:
-                    opt_func = func
-                    opt_params = params
-                    min_mse = mse
-            assert min_mse is not None, "Curve fit failed to find a suitable regression model."
-        else:
-            opt_func = regression_functions[self.curvetype]
-            init = regression_init[self.curvetype](X, y)
-            try:
-                opt_params = curve_fit(opt_func, X, y, p0=init, maxfev = 500 * len(init))[0]
-            except (RuntimeError, TypeError, ValueError):
-                raise RuntimeError("Curve fit failed to fit regression model.")
+                    raise RuntimeError("Curve fit failed to fit regression model.")
         
-        reg = lambda x: np.maximum(np.minimum(opt_func(x, *opt_params), np.max(self.residuals)), np.min(self.residuals))
-        return reg, X, y
+            reg = lambda x: np.clip(opt_func(x, *opt_params), np.min(self.residuals), np.max(self.residuals))
+            
+            self._fit_regression_params[mode] = {"used_curvetype": used_curvetype, "params": opt_params, "X": X.tolist(), "y": y.tolist()}
+            
+            return reg, X, y
+        else:
+            params = self._fit_regression_params[mode]
+            
+            return lambda x: np.clip(regression_functions[params["used_curvetype"]](x, *params["params"]), np.max(self.residuals), np.min(self.residuals)), np.array(params["X"]), np.array(params["y"])
 
     @abstractmethod
     def calculate(
@@ -1430,6 +1184,8 @@ class BaseErrorModel(BaseClass):
         if hasattr(self, "residuals"):
             d.update({"residuals": self.residuals})
             d.update({"scores": self.scores})
+            
+        d.update({"fit_regression_params": self._fit_regression_params})
         return d
 
     def _load(self, d) -> None:
@@ -1440,11 +1196,9 @@ class BaseErrorModel(BaseClass):
             self.y_train = d["y_train"]
             self.y_pred_train = d["y_pred_train"]
             if hasattr(self, "model"):
-                print("Rebuilding error model...")
                 self._build()
         if "residuals" in d.keys():
             self.residuals = d["residuals"]
             self.scores = d["scores"]
             if hasattr(self, "model"):
-                print("Refitting error model...")
                 self._fit(self.residuals, self.scores)
\ No newline at end of file
