@@ -29,17 +29,28 @@ Footer
 import argparse
 import pickle
 from collections import Counter
+
 import olorenchemengine as oce
 from olorenchemengine.internal import mock_imports
 
+
 class TorchVocab(object):
     """
     :property freqs: collections.Counter, コーパス中の単語の出現頻度を保持するオブジェクト
     :property stoi: collections.defaultdict, string → id の対応を示す辞書
     :property itos: collections.defaultdict, id → string の対応を示す辞書
     """
-    def __init__(self, counter, max_size=None, min_freq=1, specials=['<pad>', '<oov>'],
-                 vectors=None, unk_init=None, vectors_cache=None):
+
+    def __init__(
+        self,
+        counter,
+        max_size=None,
+        min_freq=1,
+        specials=["<pad>", "<oov>"],
+        vectors=None,
+        unk_init=None,
+        vectors_cache=None,
+    ):
         """
         :param counter: collections.Counter, データ中に含まれる単語の頻度を計測するためのcounter
         :param max_size: int, vocabularyの最大のサイズ. Noneの場合は最大値なし. defaultはNone
@@ -109,7 +120,12 @@ class Vocab(TorchVocab):
         self.eos_index = 2
         self.sos_index = 3
         self.mask_index = 4
-        super().__init__(counter, specials=["<pad>", "<unk>", "<eos>", "<sos>", "<mask>"], max_size=max_size, min_freq=min_freq)
+        super().__init__(
+            counter,
+            specials=["<pad>", "<unk>", "<eos>", "<sos>", "<mask>"],
+            max_size=max_size,
+            min_freq=min_freq,
+        )
 
     # override用
     def to_seq(self, sentece, seq_len, with_eos=False, with_sos=False) -> list:
@@ -120,7 +136,7 @@ class Vocab(TorchVocab):
         pass
 
     @staticmethod
-    def load_vocab(vocab_path: str) -> 'Vocab':
+    def load_vocab(vocab_path: str) -> "Vocab":
         with open(vocab_path, "rb") as f:
             return pickle.load(f)
 
@@ -144,7 +160,9 @@ class WordVocab(Vocab):
                 counter[word] += 1
         super().__init__(counter, max_size=max_size, min_freq=min_freq)
 
-    def to_seq(self, sentence, seq_len=None, with_eos=False, with_sos=False, with_len=False):
+    def to_seq(
+        self, sentence, seq_len=None, with_eos=False, with_sos=False, with_len=False
+    ):
         if isinstance(sentence, str):
             sentence = sentence.split()
 
@@ -167,11 +185,11 @@ class WordVocab(Vocab):
         return (seq, origin_seq_len) if with_len else seq
 
     def from_seq(self, seq, join=False, with_pad=False):
-        words = [self.itos[idx]
-                 if idx < len(self.itos)
-                 else "<%d>" % idx
-                 for idx in seq
-                 if not with_pad or idx != self.pad_index]
+        words = [
+            self.itos[idx] if idx < len(self.itos) else "<%d>" % idx
+            for idx in seq
+            if not with_pad or idx != self.pad_index
+        ]
 
         return " ".join(words) if join else words
 
@@ -180,22 +198,17 @@ class WordVocab(Vocab):
         with open(vocab_path, "rb") as f:
             return pickle.load(f)
 
+
+import math
+
 import numpy as np
 import pandas as pd
-
-try:
-    import torch
-    from torch import nn
-    from torch import optim
-    from torch.autograd import Variable
-    from torch.nn import functional as F
-    from torch.utils.data import DataLoader
-except ImportError:
-    mock_imports(globals(), "torch", "nn", "optim", "Variable", "F", "DataLoader")
-
-
+import torch
+from torch import nn, optim
+from torch.autograd import Variable
+from torch.nn import functional as F
+from torch.utils.data import DataLoader
 from tqdm import tqdm
-import math
 
 PAD = 0
 UNK = 1
@@ -203,26 +216,30 @@ EOS = 2
 SOS = 3
 MASK = 4
 
+
 class PositionalEncoding(nn.Module):
     "Implement the PE function. No batch support?"
+
     def __init__(self, d_model, dropout, max_len=5000):
         super(PositionalEncoding, self).__init__()
         self.dropout = nn.Dropout(p=dropout)
 
         # Compute the positional encodings once in log space.
-        pe = torch.zeros(max_len, d_model) # (T,H)
-        position = torch.arange(0., max_len).unsqueeze(1)
-        div_term = torch.exp(torch.arange(0., d_model, 2) * -(math.log(10000.0) / d_model))
+        pe = torch.zeros(max_len, d_model)  # (T,H)
+        position = torch.arange(0.0, max_len).unsqueeze(1)
+        div_term = torch.exp(
+            torch.arange(0.0, d_model, 2) * -(math.log(10000.0) / d_model)
+        )
         pe[:, 0::2] = torch.sin(position * div_term)
         pe[:, 1::2] = torch.cos(position * div_term)
         pe = pe.unsqueeze(0)
-        self.register_buffer('pe', pe)
+        self.register_buffer("pe", pe)
 
     def forward(self, x):
-        x = x + Variable(self.pe[:, :x.size(1)],
-                         requires_grad=False)
+        x = x + Variable(self.pe[:, : x.size(1)], requires_grad=False)
         return self.dropout(x)
 
+
 class TrfmSeq2seq(nn.Module):
     def __init__(self, in_size, hidden_size, out_size, n_layers, dropout=0.1):
         super(TrfmSeq2seq, self).__init__()
@@ -234,8 +251,13 @@ class TrfmSeq2seq(nn.Module):
         self.pe = PositionalEncoding(hidden_size, dropout)
         self.pe.to(oce.CONFIG["DEVICE"])
 
-        self.trfm = nn.Transformer(d_model=hidden_size, nhead=4,
-            num_encoder_layers=n_layers, num_decoder_layers=n_layers, dim_feedforward=hidden_size)
+        self.trfm = nn.Transformer(
+            d_model=hidden_size,
+            nhead=4,
+            num_encoder_layers=n_layers,
+            num_decoder_layers=n_layers,
+            dim_feedforward=hidden_size,
+        )
         self.trfm.to(oce.CONFIG["DEVICE"])
 
         self.out = nn.Linear(hidden_size, out_size)
@@ -244,180 +266,209 @@ class TrfmSeq2seq(nn.Module):
     def forward(self, src):
         # src: (T,B)
         embedded = self.embed(src)  # (T,B,H)
-        embedded = self.pe(embedded) # (T,B,H)
-        hidden = self.trfm(embedded, embedded) # (T,B,H)
-        out = self.out(hidden) # (T,B,V)
-        out = F.log_softmax(out, dim=2) # (T,B,V)
-        return out # (T,B,V)
+        embedded = self.pe(embedded)  # (T,B,H)
+        hidden = self.trfm(embedded, embedded)  # (T,B,H)
+        out = self.out(hidden)  # (T,B,V)
+        out = F.log_softmax(out, dim=2)  # (T,B,V)
+        return out  # (T,B,V)
 
     def _encode(self, src):
         # src: (T,B)
         embedded = self.embed(src)  # (T,B,H)
-        embedded = self.pe(embedded) # (T,B,H)
+        embedded = self.pe(embedded)  # (T,B,H)
         output = embedded
         for i in range(self.trfm.encoder.num_layers - 1):
             output = self.trfm.encoder.layers[i](output, None)  # (T,B,H)
         penul = output.detach().cpu().numpy()
         output = self.trfm.encoder.layers[-1](output, None)  # (T,B,H)
         if self.trfm.encoder.norm:
-            output = self.trfm.encoder.norm(output) # (T,B,H)
+            output = self.trfm.encoder.norm(output)  # (T,B,H)
         output = output.detach().cpu().numpy()
         # mean, max, first*2
-        return np.hstack([np.mean(output, axis=0), np.max(output, axis=0), output[0,:,:], penul[0,:,:] ]) # (B,4H)
+        return np.hstack(
+            [
+                np.mean(output, axis=0),
+                np.max(output, axis=0),
+                output[0, :, :],
+                penul[0, :, :],
+            ]
+        )  # (B,4H)
 
     def encode(self, src):
         # src: (T,B)
         batch_size = src.shape[1]
-        if batch_size<=100:
+        if batch_size <= 100:
             return self._encode(src)
-        else: # Batch is too large to load
-            print('There are {:d} molecules. It will take a little time.'.format(batch_size))
-            st,ed = 0,100
-            out = self._encode(src[:,st:ed]) # (B,4H)
-            while ed<batch_size:
+        else:  # Batch is too large to load
+            print(
+                "There are {:d} molecules. It will take a little time.".format(
+                    batch_size
+                )
+            )
+            st, ed = 0, 100
+            out = self._encode(src[:, st:ed])  # (B,4H)
+            while ed < batch_size:
                 st += 100
                 ed += 100
-                out = np.concatenate([out, self._encode(src[:,st:ed])], axis=0)
+                out = np.concatenate([out, self._encode(src[:, st:ed])], axis=0)
             return out
 
-try:
-    from torch.utils.data import Dataset
-except ImportError:
-    mock_imports(globals(), "Dataset")
+
+from torch.utils.data import Dataset
+
 
 def split(sm):
-    '''
+    """
     function: Split SMILES into words. Care for Cl, Br, Si, Se, Na etc.
     input: A SMILES
     output: A string with space between words
-    '''
+    """
     arr = []
     i = 0
-    while i < len(sm)-1:
-        if not sm[i] in ['%', 'C', 'B', 'S', 'N', 'R', 'X', 'L', 'A', 'M', \
-                        'T', 'Z', 's', 't', 'H', '+', '-', 'K', 'F']:
+    while i < len(sm) - 1:
+        if not sm[i] in [
+            "%",
+            "C",
+            "B",
+            "S",
+            "N",
+            "R",
+            "X",
+            "L",
+            "A",
+            "M",
+            "T",
+            "Z",
+            "s",
+            "t",
+            "H",
+            "+",
+            "-",
+            "K",
+            "F",
+        ]:
             arr.append(sm[i])
             i += 1
-        elif sm[i]=='%':
-            arr.append(sm[i:i+3])
+        elif sm[i] == "%":
+            arr.append(sm[i : i + 3])
             i += 3
-        elif sm[i]=='C' and sm[i+1]=='l':
-            arr.append(sm[i:i+2])
+        elif sm[i] == "C" and sm[i + 1] == "l":
+            arr.append(sm[i : i + 2])
             i += 2
-        elif sm[i]=='C' and sm[i+1]=='a':
-            arr.append(sm[i:i+2])
+        elif sm[i] == "C" and sm[i + 1] == "a":
+            arr.append(sm[i : i + 2])
             i += 2
-        elif sm[i]=='C' and sm[i+1]=='u':
-            arr.append(sm[i:i+2])
+        elif sm[i] == "C" and sm[i + 1] == "u":
+            arr.append(sm[i : i + 2])
             i += 2
-        elif sm[i]=='B' and sm[i+1]=='r':
-            arr.append(sm[i:i+2])
+        elif sm[i] == "B" and sm[i + 1] == "r":
+            arr.append(sm[i : i + 2])
             i += 2
-        elif sm[i]=='B' and sm[i+1]=='e':
-            arr.append(sm[i:i+2])
+        elif sm[i] == "B" and sm[i + 1] == "e":
+            arr.append(sm[i : i + 2])
             i += 2
-        elif sm[i]=='B' and sm[i+1]=='a':
-            arr.append(sm[i:i+2])
+        elif sm[i] == "B" and sm[i + 1] == "a":
+            arr.append(sm[i : i + 2])
             i += 2
-        elif sm[i]=='B' and sm[i+1]=='i':
-            arr.append(sm[i:i+2])
+        elif sm[i] == "B" and sm[i + 1] == "i":
+            arr.append(sm[i : i + 2])
             i += 2
-        elif sm[i]=='S' and sm[i+1]=='i':
-            arr.append(sm[i:i+2])
+        elif sm[i] == "S" and sm[i + 1] == "i":
+            arr.append(sm[i : i + 2])
             i += 2
-        elif sm[i]=='S' and sm[i+1]=='e':
-            arr.append(sm[i:i+2])
+        elif sm[i] == "S" and sm[i + 1] == "e":
+            arr.append(sm[i : i + 2])
             i += 2
-        elif sm[i]=='S' and sm[i+1]=='r':
-            arr.append(sm[i:i+2])
+        elif sm[i] == "S" and sm[i + 1] == "r":
+            arr.append(sm[i : i + 2])
             i += 2
-        elif sm[i]=='N' and sm[i+1]=='a':
-            arr.append(sm[i:i+2])
+        elif sm[i] == "N" and sm[i + 1] == "a":
+            arr.append(sm[i : i + 2])
             i += 2
-        elif sm[i]=='N' and sm[i+1]=='i':
-            arr.append(sm[i:i+2])
+        elif sm[i] == "N" and sm[i + 1] == "i":
+            arr.append(sm[i : i + 2])
             i += 2
-        elif sm[i]=='R' and sm[i+1]=='b':
-            arr.append(sm[i:i+2])
+        elif sm[i] == "R" and sm[i + 1] == "b":
+            arr.append(sm[i : i + 2])
             i += 2
-        elif sm[i]=='R' and sm[i+1]=='a':
-            arr.append(sm[i:i+2])
+        elif sm[i] == "R" and sm[i + 1] == "a":
+            arr.append(sm[i : i + 2])
             i += 2
-        elif sm[i]=='X' and sm[i+1]=='e':
-            arr.append(sm[i:i+2])
+        elif sm[i] == "X" and sm[i + 1] == "e":
+            arr.append(sm[i : i + 2])
             i += 2
-        elif sm[i]=='L' and sm[i+1]=='i':
-            arr.append(sm[i:i+2])
+        elif sm[i] == "L" and sm[i + 1] == "i":
+            arr.append(sm[i : i + 2])
             i += 2
-        elif sm[i]=='A' and sm[i+1]=='l':
-            arr.append(sm[i:i+2])
+        elif sm[i] == "A" and sm[i + 1] == "l":
+            arr.append(sm[i : i + 2])
             i += 2
-        elif sm[i]=='A' and sm[i+1]=='s':
-            arr.append(sm[i:i+2])
+        elif sm[i] == "A" and sm[i + 1] == "s":
+            arr.append(sm[i : i + 2])
             i += 2
-        elif sm[i]=='A' and sm[i+1]=='g':
-            arr.append(sm[i:i+2])
+        elif sm[i] == "A" and sm[i + 1] == "g":
+            arr.append(sm[i : i + 2])
             i += 2
-        elif sm[i]=='A' and sm[i+1]=='u':
-            arr.append(sm[i:i+2])
+        elif sm[i] == "A" and sm[i + 1] == "u":
+            arr.append(sm[i : i + 2])
             i += 2
-        elif sm[i]=='M' and sm[i+1]=='g':
-            arr.append(sm[i:i+2])
+        elif sm[i] == "M" and sm[i + 1] == "g":
+            arr.append(sm[i : i + 2])
             i += 2
-        elif sm[i]=='M' and sm[i+1]=='n':
-            arr.append(sm[i:i+2])
+        elif sm[i] == "M" and sm[i + 1] == "n":
+            arr.append(sm[i : i + 2])
             i += 2
-        elif sm[i]=='T' and sm[i+1]=='e':
-            arr.append(sm[i:i+2])
+        elif sm[i] == "T" and sm[i + 1] == "e":
+            arr.append(sm[i : i + 2])
             i += 2
-        elif sm[i]=='Z' and sm[i+1]=='n':
-            arr.append(sm[i:i+2])
+        elif sm[i] == "Z" and sm[i + 1] == "n":
+            arr.append(sm[i : i + 2])
             i += 2
-        elif sm[i]=='s' and sm[i+1]=='i':
-            arr.append(sm[i:i+2])
+        elif sm[i] == "s" and sm[i + 1] == "i":
+            arr.append(sm[i : i + 2])
             i += 2
-        elif sm[i]=='s' and sm[i+1]=='e':
-            arr.append(sm[i:i+2])
+        elif sm[i] == "s" and sm[i + 1] == "e":
+            arr.append(sm[i : i + 2])
             i += 2
-        elif sm[i]=='t' and sm[i+1]=='e':
-            arr.append(sm[i:i+2])
+        elif sm[i] == "t" and sm[i + 1] == "e":
+            arr.append(sm[i : i + 2])
             i += 2
-        elif sm[i]=='H' and sm[i+1]=='e':
-            arr.append(sm[i:i+2])
+        elif sm[i] == "H" and sm[i + 1] == "e":
+            arr.append(sm[i : i + 2])
             i += 2
-        elif sm[i]=='+' and sm[i+1]=='2':
-            arr.append(sm[i:i+2])
+        elif sm[i] == "+" and sm[i + 1] == "2":
+            arr.append(sm[i : i + 2])
             i += 2
-        elif sm[i]=='+' and sm[i+1]=='3':
-            arr.append(sm[i:i+2])
+        elif sm[i] == "+" and sm[i + 1] == "3":
+            arr.append(sm[i : i + 2])
             i += 2
-        elif sm[i]=='+' and sm[i+1]=='4':
-            arr.append(sm[i:i+2])
+        elif sm[i] == "+" and sm[i + 1] == "4":
+            arr.append(sm[i : i + 2])
             i += 2
-        elif sm[i]=='-' and sm[i+1]=='2':
-            arr.append(sm[i:i+2])
+        elif sm[i] == "-" and sm[i + 1] == "2":
+            arr.append(sm[i : i + 2])
             i += 2
-        elif sm[i]=='-' and sm[i+1]=='3':
-            arr.append(sm[i:i+2])
+        elif sm[i] == "-" and sm[i + 1] == "3":
+            arr.append(sm[i : i + 2])
             i += 2
-        elif sm[i]=='-' and sm[i+1]=='4':
-            arr.append(sm[i:i+2])
+        elif sm[i] == "-" and sm[i + 1] == "4":
+            arr.append(sm[i : i + 2])
             i += 2
-        elif sm[i]=='K' and sm[i+1]=='r':
-            arr.append(sm[i:i+2])
+        elif sm[i] == "K" and sm[i + 1] == "r":
+            arr.append(sm[i : i + 2])
             i += 2
-        elif sm[i]=='F' and sm[i+1]=='e':
-            arr.append(sm[i:i+2])
+        elif sm[i] == "F" and sm[i + 1] == "e":
+            arr.append(sm[i : i + 2])
             i += 2
         else:
             arr.append(sm[i])
             i += 1
-    if i == len(sm)-1:
+    if i == len(sm) - 1:
         arr.append(sm[i])
-    return ' '.join(arr)
+    return " ".join(arr)
+
 
 class Seq2seqDataset(Dataset):
-
     def __init__(self, smiles, vocab, seq_len=220):
         self.smiles = smiles
         self.vocab = vocab
@@ -431,6 +482,6 @@ class Seq2seqDataset(Dataset):
         sm = split(sm).split()
         content = [self.vocab.stoi.get(token, self.vocab.unk_index) for token in sm]
         X = [self.vocab.sos_index] + content + [self.vocab.eos_index]
-        padding = [self.vocab.pad_index]*(self.seq_len - len(X))
+        padding = [self.vocab.pad_index] * (self.seq_len - len(X))
         X.extend(padding)
-        return torch.tensor(X)
\ No newline at end of file
+        return torch.tensor(X)
