@@ -81,11 +81,15 @@ class MolCLR(BaseModel):
         epochs=100,
         batch_size=32,
         init_lr=0.0005,
-        init_base_lr=0.0001,
+        init_base_lr=None,
+        base_coef = 1,
         weight_decay=1e-6,
         **kwargs,
     ):
         self.model_type = model_type
+        
+        if init_base_lr is None:
+            init_base_lr = init_lr/base_coef
 
         self.config = {
             "epochs": epochs,
@@ -246,6 +250,7 @@ class MolCLRVecRep(BaseVecRepresentation):
         init_lr=0.0005,
         init_base_lr=0.0001,
         weight_decay=1e-6,
+        log=True,
         **kwargs,
     ):
         self.model_type = model_type
@@ -284,6 +289,8 @@ class MolCLRVecRep(BaseVecRepresentation):
             torch.load(save_path, map_location=oce.CONFIG["MAP_LOCATION"])
         )
         self.model.to(oce.CONFIG["DEVICE"])
+        
+        super().__init__(log=False,**kwargs)
 
     def _convert(self, X, **kwargs):
         return self.convert(X)
