@@ -76,6 +76,10 @@ class BaseModelManager(BaseClass):
 
     def primary_metric(self):
         return self.primary_metric
+    
+    @property
+    def direction(self):
+        return metric_direction[self.primary_metric] # "higher" or "lower"
 
     def run(
         self, models: Union[BaseModel, List[BaseModel]], return_models: bool = False
@@ -184,9 +188,14 @@ class BaseModelManager(BaseClass):
                 return primary_metric_outputs
 
     def get_model_database(self):
-
-        return self.model_database
-
+        if self.direction == "higher":
+            return self.model_database.sort_values(
+                self.primary_metric, ascending=False
+            ).reset_index(drop=True)
+        else:
+            return self.model_database.sort_values(
+                self.primary_metric, ascending=True
+            ).reset_index(drop=True)
 
 class ModelManager(BaseModelManager):
     """ModelManager is the class that tracks model development against a specified
