@@ -14,168 +14,6 @@ import olorenchemengine as oce
 from .base_class import *
 from .dataset import *
 
-
-def get_all_reps():
-    return list(oce.all_subclasses(BaseRepresentation))
-
-
-class BaseRepresentation(BaseClass):
-
-    """BaseClass for all molecular representations (PyTorch Geometric graphs, descriptors, fingerprints, etc.)
-
-    Parameters:
-        log (boolean): Whether to log the representation or not
-
-    Methods:
-        _convert(smiles: str, y: Union[int, float, np.number] = None) -> Any: converts a single structure (represented by a SMILES string) to a representation
-        convert(Xs: Union[list, pd.DataFrame, dict, str], ys: Union[list, pd.Series, np.ndarray]=None) -> List[Any]: converts input data to a list of representations
-    """
-
-    @log_arguments
-    def __init__(self, log=True):
-        pass
-
-    @abstractmethod
-    def _convert(self, smiles: str, y: Union[int, float, np.number] = None) -> Any:
-        """Converts a single structure (represented by a SMILES string) to a representation
-
-        Parameters:
-            smiles (str): SMILES string of the structure
-            y (Union[int, float, np.number]): target value of the structure
-
-        Returns:
-            Any: representation of the structure
-        """
-        pass
-
-    def _convert_list(
-        self, smiles_list: List[str], ys: List[Union[int, float, np.number]] = None
-    ) -> List[Any]:
-        """Converts a list of structures (represented by a SMILES string) to a list of representations
-
-        Parameters:
-            smiles_list (List[str]): list of SMILES strings of the structures
-            ys (List[Union[int, float, np.number]]): list of target values of the structures
-
-        Returns:
-            List[Any]: list of representations of the structures"""
-        if ys is None:
-            return [self._convert(s) for s in smiles_list]
-            # with mp.Pool(mp.cpu_count()) as p:
-            #   X = p.map(self._convert, smiles_list)
-            # return X
-        else:
-            return [self._convert(s, y=y) for s, y in tqdm(zip(smiles_list, ys))]
-
-    def _convert_cache(
-        self, smiles: str, y: Union[int, float, np.number] = None
-    ) -> Any:
-        """Converts a single structure (represented by a SMILES string) to a representation
-
-        Parameters:
-            smiles (str): SMILES string of the structure
-            y (Union[int, float, np.number]): target value of the structure
-        Returns:
-            Any: representation of the structure
-        """
-        if smiles in self.cache.keys():
-            return self.cache[smiles]
-        else:
-            return self._convert(smiles, y=y)
-
-    def convert(
-        self,
-        Xs: Union[list, pd.DataFrame, dict, str],
-        ys: Union[list, pd.Series, np.ndarray] = None,
-        **kwargs,
-    ) -> List[Any]:
-        """Converts input data to a list of representations
-
-        Parameters:
-            Xs (Union[list, pd.DataFrame, dict, str]): input data
-            ys (Union[list, pd.Series, np.ndarray]=None): target values of the input data
-
-        Returns:
-            List[Any]: list of representations of the input data
-        """
-        if isinstance(Xs, list) and (
-            isinstance(Xs[0], list) or isinstance(Xs[0], tuple)
-        ):
-            smiles = [X[0] for X in Xs]
-        elif isinstance(Xs, pd.DataFrame) or isinstance(Xs, dict):
-            if isinstance(Xs, pd.DataFrame):
-                keys = Xs.columns.tolist()
-            elif isinstance(Xs, dict):
-                keys = Xs.keys()
-            if "smiles" in keys:
-                smiles = Xs["smiles"]
-            elif "Smiles" in keys:
-                smiles = Xs["Smiles"]
-            elif "SMILES" in keys:
-                smiles = Xs["SMILES"]
-            else:
-                smiles = Xs.iloc[:, 0].tolist()
-        elif isinstance(Xs, str):
-            smiles = [Xs]
-        else:
-            smiles = Xs
-
-        return self._convert_list(smiles, ys=ys)
-
-    def _save(self):
-        return {}
-
-    def _load(self, d):
-        pass
-
-
-class SMILESRepresentation(BaseRepresentation):
-    """Extracts the SMILES strings from inputted data
-
-    Methods:
-        convert(Xs: Union[list, pd.DataFrame, dict, str], ys: Union[list, pd.Series, np.ndarray]=None) -> List[Any]: converts input data to a list of SMILES strings
-            Data types:
-                pd.DataFrames will have columns "smiles" or "Smiles" or "SMILES" extracted
-                lists and tuples of multiple elements will have their first element extracted
-                strings will be converted to a list of one element
-                everything else will be returned as inputted
-    """
-
-    def _convert(self, smiles, y=None):
-        pass
-
-    def convert(self, Xs, ys=None, **kwargs):
-        if isinstance(Xs, list) and (
-            isinstance(Xs[0], list) or isinstance(Xs[0], tuple)
-        ):
-            smiles = [X[0] for X in Xs]
-        elif isinstance(Xs, pd.DataFrame) or isinstance(Xs, dict):
-            if isinstance(Xs, pd.DataFrame):
-                keys = Xs.columns.tolist()
-            elif isinstance(Xs, dict):
-                keys = Xs.keys()
-            if "smiles" in keys:
-                smiles = Xs["smiles"]
-            elif "Smiles" in keys:
-                smiles = Xs["Smiles"]
-            elif "SMILES" in keys:
-                smiles = Xs["SMILES"]
-            elif "smi" in keys:
-                smiles = Xs["smi"]
-            else:
-                smiles = Xs.iloc[:, 0]
-        elif isinstance(Xs, str):
-            smiles = [Xs]
-        elif isinstance(Xs, pd.Series):
-            smiles = Xs.tolist()
-        else:
-            smiles = Xs
-
-        if isinstance(smiles, pd.Series):
-            smiles = smiles.tolist()
-        return smiles
-
-
 class AtomFeaturizer(BaseClass):
     """Abstract class for atom featurizers, which create a vector representation for a single atom.
 
@@ -417,256 +255,6 @@ class TorchGeometricGraph(BaseRepresentation):
         self.atom_featurizer._load(d["atom_featurizer"])
         self.bond_featurizer._load(d["bond_featurizer"])
 
-
-class BaseVecRepresentation(BaseRepresentation):
-    """Representation where given input data, returns a vector representation for each compound."""
-
-    @log_arguments
-    def __init__(
-        self,
-        *args,
-        collinear_thresh=1.01,
-        scale=StandardScaler(),
-        names=None,
-        log=True,
-        **kwargs,
-    ):
-        self.collinear_thresh = collinear_thresh
-        self.to_drop = None
-        if not scale is None:
-            scale = scale.copy()
-        self.scale = scale
-        self.scale_fitted = False
-        self._names = names
-
-        from os import path
-
-        if not path.exists(
-            path.join(
-                oce.CONFIG["CACHE_PATH"], f"vecrep/{self.__class__.__name__}/"
-            )
-        ):
-            os.mkdir(
-                path.join(
-                oce.CONFIG["CACHE_PATH"], f"vecrep/{self.__class__.__name__}/"
-                )
-            )
-
-        super().__init__(*args, log=False, **kwargs)
-
-    @property
-    def names(self):
-        if not self._names is None:
-            return self._names
-        else:
-            raise ValueError(
-                f"Names not set for representation {self.__class__.__name__}"
-            )
-
-    def convert(
-        self,
-        Xs: Union[list, pd.DataFrame, dict, str],
-        ys: Union[list, pd.Series, np.ndarray] = None,
-        fit=False,
-        **kwargs,
-    ) -> List[np.ndarray]:
-        """BaseVecRepresentation's convert returns a list of numpy arrays.
-
-        Args:
-            Xs (Union[list, pd.DataFrame, dict, str]): input data
-            ys (Union[list, pd.Series, np.ndarray], optional): included for compatibility, unused argument. Defaults to None.
-
-        Returns:
-            List[np.ndarray]: list of molecular vector representations
-        """
-        import joblib
-
-        input_hash = (
-            joblib.hash(Xs)
-            + joblib.hash(ys)
-            + joblib.hash(self._save())
-            + joblib.hash(oce.parameterize(self))
-        )
-
-        from os import path
-
-        if oce.CONFIG["CACHE"] and path.exists(
-            path.join(
-                oce.CONFIG["CACHE_PATH"],
-                f"vecrep/{self.__class__.__name__}/{input_hash}.npy",
-            )
-        ):
-            return np.load(
-                path.join(
-                    oce.CONFIG["CACHE_PATH"],
-                    f"vecrep/{self.__class__.__name__}/{input_hash}.npy",
-                ),
-                allow_pickle=True,
-            )
-
-        feats = super().convert(Xs, ys)
-        import pandas as pd
-
-        feats = pd.DataFrame.from_records(
-            feats, columns=[f"col{i}" for i in range(len(feats[0]))]
-        )
-        if fit and len(Xs) > 2:
-            # collinear
-            corr_matrix = feats.corr().abs()
-            upper = corr_matrix.where(
-                np.triu(np.ones(corr_matrix.shape), k=1).astype(np.bool)
-            )
-            self.to_drop = [
-                column
-                for column in upper.columns
-                if any(upper[column] > self.collinear_thresh)
-            ]
-            feats = feats.drop(columns=self.to_drop)
-            # scale
-            if not self.scale is None:
-                x = self.scale.fit_transform(feats.values)
-                feats.values[:] = np.nan_to_num(x.reshape(feats.values.shape))
-                self.scale_fitted = True
-        else:
-            if not self.to_drop is None:
-                feats = feats.drop(columns=self.to_drop)
-            # scale
-            if self.scale_fitted and self.scale is not None:
-                x = self.scale.transform(feats.values)
-                feats.values[:] = np.nan_to_num(x.reshape(feats.values.shape))
-
-        output = np.nan_to_num(np.array(feats.to_records(index=False).tolist()))
-        if oce.CONFIG["CACHE"]:
-            np.save(
-                path.join(
-                    oce.CONFIG["CACHE_PATH"],
-                    f"vecrep/{self.__class__.__name__}/{input_hash}.npy",
-                ),
-                output,
-                allow_pickle=True,
-            )
-        return output
-
-    def calculate_distance(
-        self,
-        x1: Union[str, List[str]],
-        x2: Union[str, List[str]],
-        metric: str = "cosine",
-        **kwargs,
-    ) -> np.ndarray:
-        """Calculates the distance between two molecules or list of molecules.
-
-        Returns a 2D array of distances between each pair of molecules of shape
-        len(x1) by len(x2).
-
-        This uses pairwise_distances from sklearn.metrics to calculate distances
-        between the vector representations of the molecules. Options for distances
-        are Valid values for metric are:
-
-            From scikit-learn: [‘cityblock’, ‘cosine’, ‘euclidean’, ‘l1’, ‘l2’,
-                ‘manhattan’]. These metrics support sparse matrix inputs.
-                [‘nan_euclidean’] but it does not yet support sparse matrices.
-            From scipy.spatial.distance: [‘braycurtis’, ‘canberra’, ‘chebyshev’,
-                ‘correlation’, ‘dice’, ‘hamming’, ‘jaccard’, ‘kulsinski’,
-                ‘mahalanobis’, ‘minkowski’, ‘rogerstanimoto’, ‘russellrao’,
-                ‘seuclidean’, ‘sokalmichener’, ‘sokalsneath’, ‘sqeuclidean’,
-                ‘yule’].
-
-            See the documentation for scipy.spatial.distance for details on these metrics.
-        """
-
-        from sklearn.metrics import pairwise_distances
-
-        if isinstance(x1, str):
-            x1 = [x1]
-        if isinstance(x2, str):
-            x2 = [x2]
-        x1 = self.convert(x1)
-        x2 = self.convert(x2)
-        return pairwise_distances(x1, x2, metric=metric, **kwargs)
-
-    def __add__(self, other):
-        """Adds two representations together
-
-        Parameters:
-            other (BaseVecRepresentation): representation to add to the current representation
-
-        Returns:
-            BaseVecRepresentation: new representation"""
-        return ConcatenatedVecRepresentation(self, other)
-
-    def _save(self):
-        d = {}
-        d.update({"collinear_thresh": self.collinear_thresh})
-        d.update({"to_drop": self.to_drop})
-        if not self.scale is None:
-            d.update({"scale": self.scale._save()})
-            d.update({"scale_fitted": self.scale_fitted})
-        return d
-
-    def _load(self, d):
-        self.collinear_thresh = d["collinear_thresh"]
-        self.to_drop = d["to_drop"]
-        if not self.to_drop is None:
-            self.to_drop = set(self.to_drop)
-        if not self.scale is None and "scale" in d:
-            self.scale._load(d["scale"])
-            self.scale_fitted = d["scale_fitted"]
-
-
-class ConcatenatedVecRepresentation(BaseVecRepresentation):
-    """Creates a structure vector representation by concatenating multiple representations.
-
-    Parameters:
-        rep1 (BaseVecRepresentation): first representation to concatenate
-        rep2 (BaseVecRepresentation): second representation to concatenate
-        log (bool): whether to log the representations or not
-
-    Can be created by adding two representations together using + operator.
-
-    Example
-    ------------------------------
-    import olorenchemengine as oce
-    combo_rep = oce.MorganVecRepresentation(radius=2, nbits=2048) + oce.Mol2Vec()
-    model = oce.RandomForestModel(representation = combo_rep, n_estimators = 1000)
-
-    model.fit(train['Drug'], train['Y'])
-    model.predict(test['Drug'])
-    ------------------------------
-    """
-
-    @log_arguments
-    def __init__(
-        self,
-        rep1: BaseVecRepresentation,
-        rep2: BaseVecRepresentation,
-        log=True,
-        **kwargs,
-    ):
-        self.rep1 = rep1
-        self.rep2 = rep2
-        try:
-            self._names = rep1.names + rep2.names
-        except Exception as e:
-            self._names = None
-        super().__init__(names=self._names, log=False, **kwargs)
-
-    def _convert(self, smiles, y=None, fit=False):
-        converted_1 = self.rep1._convert(smiles, y=y, fit=fit)
-        converted_2 = self.rep2._convert(smiles, y=y, fit=fit)
-        return np.concatenate((converted_1, converted_2))
-
-    def _convert_list(self, smiles_list, ys=None, fit=False):
-        converted_1 = self.rep1._convert_list(smiles_list, ys=ys, fit=fit)
-        converted_2 = self.rep2._convert_list(smiles_list, ys=ys, fit=fit)
-        return np.concatenate((converted_1, converted_2), axis=1)
-
-    def convert(self, smiles_list, ys=None, fit=False, **kwargs):
-        converted_1 = self.rep1.convert(smiles_list, ys=ys, fit=fit)
-        converted_2 = self.rep2.convert(smiles_list, ys=ys, fit=fit)
-        return np.concatenate((converted_1, converted_2), axis=1)
-
-
 class NoisyVec(BaseVecRepresentation):
     """Adds noise to a given BaseVecRepresentation
 
@@ -751,11 +339,12 @@ class BaseCompoundVecRepresentation(BaseVecRepresentation):
         self,
         Xs: Union[list, pd.Series, pd.DataFrame, dict, str],
         ys: Union[list, pd.Series, np.ndarray] = None,
+        lambda_convert: Callable = None,
         fit=False,
         **kwargs,
     ) -> np.ndarray:
         """Computes a vector representation from each structure in Xs."""
-        feats = np.array(super().convert(Xs, ys, fit=fit))
+        feats = np.array(super().convert(Xs, ys, fit=fit, lambda_convert = lambda_convert))
         out = np.nan_to_num(feats.astype(np.float32))
         return out
 
@@ -1224,7 +813,7 @@ class MordredDescriptor(BaseCompoundVecRepresentation):
     @log_arguments
     def __init__(
         self,
-        descriptor_set: Union[str, list] = "all",
+        descriptor_set: Union[str, list] = "2d",
         log: bool = True,
         normalize: bool = False,
         **kwargs,
@@ -1235,6 +824,8 @@ class MordredDescriptor(BaseCompoundVecRepresentation):
 
         if descriptor_set == "all":
             self.calc = Calculator(descriptors, ignore_3D=False)
+        elif descriptor_set == "2d":
+            self.calc = Calculator(descriptors, ignore_3D=True)
 
         self.normalize = normalize
 
@@ -1243,7 +834,7 @@ class MordredDescriptor(BaseCompoundVecRepresentation):
     def _convert(self, smiles, y=None):
         pass
 
-    def convert(self, Xs, ys=None, **kwargs):
+    def convert_full(self, Xs, ys=None, **kwargs):
         """Convert list of SMILES to descriptors in the form of a numpy array.
 
         Parameters:
@@ -1262,6 +853,9 @@ class MordredDescriptor(BaseCompoundVecRepresentation):
             feats[col] = pd.to_numeric(feats[col], errors="coerce").fillna(0)
         feats = np.nan_to_num(feats.to_numpy())
         return feats
+    
+    def convert(self, Xs, ys=None, **kwargs):
+        return super().convert(Xs, ys=ys, lambda_convert = self.convert_full,**kwargs)
 
 
 from rdkit import Chem, DataStructs, RDConfig
